--	LibBuilder
--	A tool for building multiple-file libraries


--	TODO: Implement better tracking for when libraries aren't closed properly

local MAJOR, MINOR = "LibBuilder-1.0", 1


local LibBuilder = LibStub:NewLibrary(MAJOR, MINOR)
if not LibBuilder then return end


LibBuilder.errorStrings = {
	SHELL = "%s: %s!", 
	BADVERSION = "NewLib requires a textual major and numeric minor version", 
	COLLISION = "Attempted to build %s, rev. %s before closing %s, rev. %s", 
	NOLIB = "GetLib called but no library is being built", 
}


--	used fields are listed here for documentation purposes
LibBuilder.workspace = nil
LibBuilder.major = nil
LibBuilder.minor = nil


---	Registers a new library and creates a workspace for it.
--	@param	major		The major version of the library being registered.
--	@param	minor		The minor version of the library being registered.
--	@param	workspace	Optional, an existing table to use as the workspace.
--	@return	workspace	The workspace for the new library.
function LibBuilder:NewLib(major, minor, workspace)
	--	check the supplied version info
	if type(major) ~= "string" or type(minor) ~= "number" then
		local message = self:BuildErrorMessage("BADVERSION", major, minor)
		error(message)
	end
	
	--	check for collisions
	if self.major or self.minor or self.workspace then
		local message = self:BuildErrorMessage("COLLISION", self.major, self.minor, major, minor)
		error(message)
	end

	self.major = major
	self.minor = minor
	
	--	if the programmer gave us a funky workspace, that's their problem
	self.workspace = workspace or {}
	
	return self.workspace
end


---	Retrieves the version info and workspace for the current active library.
--	@return major		Major version of the library.
--	@return minor		Minor version of the library.
--	@return workspace	The workspace for the library.
function LibBuilder:GetLib()
	if not self.major then		--	just check major to allow for flexibility
		local message = self:BuildErrorMessage("NOLIB")
		error(message)
	end
	
	return self.major, self.minor, self.workspace
end


---	Closes the active library. This is used for tracking, to make sure 
--	that two libraries don't get clobbered together.
function LibBuilder:CloseLib()
	self.major, self.minor, self.workspace = nil, nil, nil
end


---	Private, builds an error message
--	@param	errorName	Name of the error
--	@param	...			List of arguments; length depends on the error type.
--	@return message		The complete error string.
function LibBuilder:BuildErrorMessage(errorName, ...)
	local body = self.errorStrings[errorName]:format(...)
	local message = self.errorStrings.SHELL:format(MAJOR, body)

	return message
end
